SAIKA_DEBUG = 'SAIKA_DEBUG'

# region Meta Item
MI_GLOBAL = 'global'
MI_CALLBACK = 'callback'
# endregion

# region Meta Key

# region Global Part
MK_CONTROLLER_CLASSES = 'controller_classes'
MK_MODEL_CLASSES = 'model_classes'
# endregion

# region Function Key
MK_RULE_STR = 'rule_str'
MK_METHODS = 'methods'
MK_OPTIONS = 'options'
MK_FORM_CLASS = 'form_class'
MK_FORM_ARGS = 'form_args'
MK_FORM_VALIDATE = 'form_validate'
# endregion

# region Config Key
CK_CORE = 'core'
CK_DATABASE = 'database'
CK_CORS = 'cors'
CK_SOCKET_IO = 'socket_io'
# endregion

# region Arg Key
AK_VALIDATE = 'validate'
# endregion

# region G Key
GK_KEYS = '_keys'
GK_CONTEXT = 'context'
GK_FORM = 'form'
GK_SOCKET = 'socket'
# endregion

# region Callback Key
MK_BEFORE_APP_REQUEST = 'before_app_request'
MK_BEFORE_APP_FIRST_REQUEST = 'before_app_first_request'
MK_AFTER_APP_REQUEST = 'after_app_request'
# endregion

# endregion
